﻿using System;
using System.Text.RegularExpressions;

namespace CSDemo {
  class MainClass {
    public static void Main(string[] args) {
      // quick test for the balanced match failure condition
      //Console.WriteLine(Regex.Match("abc", "(?<foo>a)(?(foo)(?!))").Success);

//      InsertCommasWithLookBehind();
      NamedCaptureAccess();
//      BalancedParens();
    }

    public static void BalancedParens() {
      var rs = @"
[^()]*                  # there might be stuff at the start
(?<formula>             # formula group captures the complete () part
(                       # this group starts a () part
  
  (                     # this is the start of an opening paren group
    [^()]*              # stuff before the opening paren
    (?<start>\()        # the opening paren itself
    [^()]*              # stuff after the opening paren
  )+                    # we can have lots of parens opening

  (                     # this is the start of the closing paren group
    (?<end-start>\))    # the closing paren itself - 
                        # the special name 'closes' the corresponding start group
  )+                    # we should have lots of parens closing, too

)*                      # the end of the nested paren groups
)                       # the end of the formula group

#(?!)                   # just for testing purposes
#(?(start)(?!))         # this should fail the expression if there aren't
                        # enough closing elements - doesn't work right though
";
      var r = new Regex(rs, RegexOptions.IgnorePatternWhitespace);

      var text = "formula: (10 * (3 + (7 - 5) + (2 + 6))) <- this is important";
      var match = r.Match(text);

      if (match.Success) {
        Console.WriteLine("Match: " + match.Value);
        Console.WriteLine("Formula: " + match.Groups["formula"]);
        Console.WriteLine("start captures:");
        var starts = match.Groups["start"];
        foreach (var start in starts.Captures) {
          Console.WriteLine(start);
        }

        Console.WriteLine("end captures:");
        var ends = match.Groups["end"];
        foreach (var end in ends.Captures) {
          Console.WriteLine(end);
        }
      }
      else
        Console.WriteLine("Match not successful");
    }

    public static void InsertCommasWithLookBehind() {
      var r = new Regex(@"(?<=\d)(?=(\d{3})+(?!\d))");
      var text = "1234567890";
      Console.WriteLine(r.Replace(text, @","));
    }

    public static void NamedCaptureAccess() {
      var r = new Regex(@"(\s*(?<items>[^;]+)\s*(;|$))*", RegexOptions.None);
      //var r_ = new Regex(@"(\s*(?<items>[^;]+)\s*(;|$))*", RegexOptions.ExplicitCapture);

      var text = "\"Text\"; 10.3; 24/2/2016 ;\"More text\"; 17";
      var match = r.Match(text);

      Console.WriteLine(match);

      var group = match.Groups["items"];
      foreach(var item in group.Captures)
        Console.WriteLine(item);
    }
  }
}
